﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;

/*
 *         Progetto: CodificheSegnale
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 11/10/2017
 *  
 *      Descrizione: Classe della finestra di dialogo contenente i controlli grafici. 
 */


namespace CodificheSegnale
{
    /// <summary>
    /// Logica di interazione per MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string buffer;
        private string SequenzaDaDisegnare
        {
            get
            {
                return buffer;
            }

            set
            {
                buffer = value;
                DisegnaNRZ();
                DisegnaRZ();
                DisegnaManchesterThomas();
                DisegnaManchesterIEEE();
            }
        }
        public MainWindow()
        {
            InitializeComponent();
        }

        private void aggiornaSequenza()
        {
            try
            {
                tbSeqCodificata.Text = "";
                Valida(tbSequenza.Text);

                if (cb4B5B.IsChecked ?? false)
                {
                    // Per applicare la codifica 4B5B il numero di bit deve essere multiplo di 4. 
                    // Pertanto si aggiungono, se necessario, uno o più simboli "0" in coda alla stringa di sequenza

                    string s = tbSequenza.Text;
                    while (s.Length % 4 != 0)
                    {
                        s += "0";
                    }
                    tbSequenza.Text = s;
                    tbSeqCodificata.Text = Codifica4B5B(s);
                }
                else
                    tbSeqCodificata.Text = tbSequenza.Text;

                SequenzaDaDisegnare = tbSeqCodificata.Text;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Errore di validazione: " + ex.Message, "Errore", MessageBoxButton.OK, MessageBoxImage.Error);
                SequenzaDaDisegnare = null;
            }
        }

        public void Valida(string sequenza)
        {
            if (sequenza.Length == 0)
                throw new Exception("La sequenza deve includere almeno un simbolo");

            for (int i = 0; i < sequenza.Length; i++)
            {
                if (sequenza[i] != '0' && sequenza[i] != '1')
                    throw new Exception("Simbolo non valido");

            }

        }

        public string Codifica4B5B(string sequenza)
        {
            // Se il numero di cifre non è un multiplo di 4, 
            // aggiungi in coda alla sequenza cifre 0 di padding

            while (sequenza.Length % 4 != 0)
                sequenza += '0';

            StringBuilder sb = new StringBuilder();
            string gruppo4, gruppo5;
            Dictionary<string, string> tabella = new Dictionary<string, string>();

            tabella.Add("0000", "11110");
            tabella.Add("0001", "01001");
            tabella.Add("0010", "10100");
            tabella.Add("0011", "10101");
            tabella.Add("0100", "01010");
            tabella.Add("0101", "01011");
            tabella.Add("0110", "01110");
            tabella.Add("0111", "01111");
            tabella.Add("1000", "10010");
            tabella.Add("1001", "10011");
            tabella.Add("1010", "10110");
            tabella.Add("1011", "10111");
            tabella.Add("1100", "11010");
            tabella.Add("1101", "11011");
            tabella.Add("1110", "11100");
            tabella.Add("1111", "11101");

            while (sequenza.Length > 0)
            {
                gruppo4 = sequenza.Substring(0, 4);
                gruppo5 = tabella[gruppo4];
                sb.Append(gruppo5);
                sequenza = sequenza.Remove(0, 4);
            }

            return sb.ToString();
        }


        private void button_Click(object sender, RoutedEventArgs e)
        {
            aggiornaSequenza();
        }

        private void DisegnaNRZ()
        {
            canvasNRZ.Children.Clear();

            if (SequenzaDaDisegnare == null)
                return;

            canvasNRZ.Width = 50 + SequenzaDaDisegnare.Length * Segnale.pixelSemiperiodo * 2;
            canvasNRZ.Height = 300;

            SegnaleClock sc = new SegnaleClock(canvasNRZ, new Point(10, 10), 80);
            sc.Codifica(SequenzaDaDisegnare);
            sc.Disegna();
            
            SegnaleNRZ s = new SegnaleNRZ(canvasNRZ, new Point(10, 90), 160);
            s.Codifica(SequenzaDaDisegnare);
            s.Disegna();

        }

        private void DisegnaRZ()
        {
            canvasRZ.Children.Clear();

            if (SequenzaDaDisegnare == null)
                return;

            canvasRZ.Width = 50 + SequenzaDaDisegnare.Length * Segnale.pixelSemiperiodo * 2;
            canvasRZ.Height = 300;

            SegnaleClock sc = new SegnaleClock(canvasRZ, new Point(10, 10), 80);
            sc.Codifica(SequenzaDaDisegnare);
            sc.Disegna();

            SegnaleRZ s = new SegnaleRZ(canvasRZ, new Point(10, 90), 160);
            s.Codifica(SequenzaDaDisegnare);
            s.Disegna();

        }

        private void DisegnaManchesterThomas()
        {
            canvasManchesterThomas.Children.Clear();

            if (SequenzaDaDisegnare == null)
                return;

            canvasManchesterThomas.Width = 50 + SequenzaDaDisegnare.Length * Segnale.pixelSemiperiodo * 2; ;
            canvasManchesterThomas.Height = 300;

            SegnaleClock sc = new SegnaleClock(canvasManchesterThomas, new Point(10, 10), 80);
            sc.Codifica(SequenzaDaDisegnare);
            sc.Disegna();

            SegnaleManchesterThomas s = new SegnaleManchesterThomas(canvasManchesterThomas, new Point(10, 90), 160);
            s.Codifica(SequenzaDaDisegnare);
            s.Disegna();

        }

        private void DisegnaManchesterIEEE()
        {
            canvasManchesterIEEE.Children.Clear();

            if (SequenzaDaDisegnare == null)
                return;

            canvasManchesterIEEE.Width = 50 + SequenzaDaDisegnare.Length * Segnale.pixelSemiperiodo * 2; ;
            canvasManchesterIEEE.Height = 300;

            SegnaleClock sc = new SegnaleClock(canvasManchesterIEEE, new Point(10, 10), 80);
            sc.Codifica(SequenzaDaDisegnare);
            sc.Disegna();

            SegnaleManchesterIEEE s = new SegnaleManchesterIEEE(canvasManchesterIEEE, new Point(10, 90), 160);
            s.Codifica(SequenzaDaDisegnare);
            s.Disegna();

        }
        private void cb4B5B_Click(object sender, RoutedEventArgs e)
        {
            aggiornaSequenza();
        }

        private void tbSequenza_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (SequenzaDaDisegnare != null)
            {
                SequenzaDaDisegnare = null;
                tbSeqCodificata.Text = "";
            }

        }

        private void tbSequenza_GotKeyboardFocus(object sender, System.Windows.Input.KeyboardFocusChangedEventArgs e)
        {
            if (sender != null)
                ((TextBox)sender).SelectAll();
        }
    }
}
