﻿using System.Windows.Media;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Shapes;

/*
 *         Progetto: CodificheSegnale
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 10/10/2017
 *  
 *      Descrizione: Definizione della classe di base da cui 
 *                   derivano tutte le classi dei segnali. 
 */

namespace CodificheSegnale
{
    public abstract class Segnale
    {
        public const int pixelSemiperiodo = 25;

        private string nome;
        protected Canvas area;
        protected Point origine;
        protected int altezza;
        protected string bufferLivelli;
        protected string bufferSequenza;
        protected bool mostraSimboli = true;


        public Segnale(Canvas area, Point origine, int altezza, string nome)
        {
            this.area = area;
            this.origine = origine;
            this.altezza = altezza;
            this.nome = nome;
        }

        //
        public abstract void Codifica(string sequenza);

        public virtual void Disegna()
        {
            double LOW = origine.Y + altezza - 1;
            double HIGH = LOW - altezza / 2;

            // assi
            Point p1 = new Point(origine.X, LOW);
            Point p2 = new Point(origine.X + bufferLivelli.Length * pixelSemiperiodo, LOW);
            TracciaLinea(area, p1, p2, 1);

            Point ultimo = p1;

            // Segnale
            for (int i = 0; i < bufferLivelli.Length; i++)
            {
                p1 = new Point(ultimo.X, (bufferLivelli[i] == 'H' ? HIGH : LOW));
                p2 = Point.Add(p1, new Vector(pixelSemiperiodo, 0));
                TracciaLinea(area, ultimo, p1, 3);
                TracciaLinea(area, p1, p2, 3);

                // Mostra cifre binarie
                if (mostraSimboli && i % 2 != 0)
                {
                    MostraTesto(
                        area,
                        new Point(p1.X - 7, LOW + 7),
                        bufferSequenza[i / 2].ToString()
                    );
                }
                ultimo = p2;
            }

            // Nome
            MostraTesto(
                area,
                new Point(origine.X + 2, HIGH - 30),
                nome
            );
        }

        public static void TracciaLinea(
            Canvas c, 
            Point p1 , 
            Point p2,
            int spessore,
            SolidColorBrush colore = null,
            bool tratteggio = false)
        {
            if (colore == null)
                colore = System.Windows.Media.Brushes.Black;

            Line ln = new Line();
            ln.Stroke = colore;
            ln.StrokeThickness = spessore;
            if (tratteggio)
                ln.StrokeDashArray = new DoubleCollection() { 4 };
            ln.StrokeStartLineCap = PenLineCap.Triangle;
            ln.StrokeEndLineCap = PenLineCap.Triangle;
            ln.StrokeLineJoin = PenLineJoin.Miter;
            // Aggiungendo 0.5 alle singole coordinate, si risolve il problema delle "blurry lines"
            ln.X1 = p1.X + 0.5;
            ln.Y1 = p1.Y + 0.5;
            ln.X2 = p2.X + 0.5;
            ln.Y2 = p2.Y + 0.5;
            c.Children.Add(ln);
        }

        public static void MostraTesto(
            Canvas c, 
            Point p, 
            string testo,
            int dimFont = 12)
        {
            TextBlock t = new TextBlock();
            t.Text = testo;
            t.Foreground = System.Windows.Media.Brushes.Black;
            t.Background = System.Windows.Media.Brushes.White;
            t.FontFamily = new FontFamily("Arial");
            t.FontSize = dimFont;
            Canvas.SetLeft(t, p.X);
            Canvas.SetTop(t, p.Y);
            c.Children.Add(t);

        }
    }
}
